'This program estimates the nonlinear and Perron models of AR(1) 
'          with a structural break, and tests series for unit root.
'          See Instructions.pdf for details.
 
'          Version: 08.2005

'          Author: Konstantin Gluschenko
'                       glu@nsu.ru
'                       http://econom.nsu.ru/users/gluschenko


'ARGUMENTS of the program may follow in any order.  All arguments are optional. If an argument is omitted, the program uses either a default value or a previous user-specified value (saved in user's workfile in table SETTINGS). 
' By default, the program uses Excel file "CriticalValues.xls" that contains critical values of the tests. User may specify a different relevant file; see Instructions.pdf for its structure.

'The arguments are as follows: 
'(<...> means some value of argument, [...] means the optional part of argument, | means "or"):

' series=<series name> is the name of a series to be analyzed. 
' type=[-]nc|c is the type of the models: "nc" means "no constant";
'          "c" means "with constant"; 
'          minus means a break to be "reversed", that is,  
'          the break dummy=1 if t<=the break point, and 0 otherwise.
'          By default, type=c. 
'break=<date1>[-<date2>[;<step>]] is the break date or the range of 
'          break dates; <date1> and <date2> should be specified as 
'          dates(according to the frequency in your workfile) 
'          rather than the observation numbers; they should represent 
'          just the actual break point, i.e., theta+1 rather than theta. 
'          If range <date1>-<date2> is specified, the program runs
'          estimations for break dates from <date1> through <date2> 
'          with increment <step>, and then takes the point that yields 
'          the  minimal sum of squared residuals. 
'          If  <date2> is omitted, the only break point <date1> is used.
'          If the argument is omitted, the first and last break points
'          available in the file of critical values are used as <date1> and 
'          <date2>. The default value of <step> is 1.
' CVfile=[<path>]<file name> is the name of an (Excel) file with
'          critical values of the tests (if there are blanks, enclose it in
'          quotes).
'          By default, CVfile=CriticalValues.xls. (It shold be placed to 
'          the default directory; otherwise specify it.)
' T=<number> is the sample size for test statistics. (Note that
'          t=0,...,T.) This argument is necessary if the file of critical 
'          values does not contain the sample size that the processed 
'          series has. Then you should specify a sample size you wish 
'          to use instead for testing.T=10000 stands for asymptotic 
'          values. By default, the sample size in your workfile 
'          (minus one) is taken.
' Perron=yes[,pt|,or]|no enables/disables estimating a Perron's
'          model; "pt" means the Perron-type model, while "or" stands 
'          for the original Perron's model. By default, Perron=no; in 
'          "yes" option, the default is "pt".
' DF=yes|no enables/disables the Dickey-Fuller and Phillips-Perron 
'          tests. By default, DF=yes.
' keep=yes|no is the indication whether to keep the program 
'          objects in user's workfile. If yes, the settings 
'          are used instead of default values unless replaced by new 
'          arguments. If keep=no, the program removes its objects
'          (including  outputs) from user's workfile. 
'          By default, keep=yes.

'EXAMPLES: 
'  series=US_GDP_detrended break=1929:01-1931:12;3 type=c T=100 Perron=yes DF=no clean=yes CVfile="D:\Projects\UR with break\Critical Values.xls" 
'  type=-nc  series=A break=98:8-99:2 Perron=yes,or
'  CVfile=CValues.xls series=R07 break=2004:4 DF=no

' Before starting the program, set "Maximum errors before halting" to a number more than 1. Then progran can run even if some information in the critical values file is missing.

'*************************** SUBROUTINES ***************************
'------------ Find an argument in the program argument list  -------------
Subroutine Find_argument(table Arguments, scalar N_of_args, string %keyword, scalar kw_length, string %argument, table Description, scalar descr_row, string %Settings)
     for !a=1 to N_of_args
            if @left(Arguments(!a,1),kw_length)=%keyword then
                 ExitLoop
            endif
     next
%argument=@mid(Arguments(!a,1),kw_length+1)
     if %argument="" and @IsObject(%Settings) then
          %argument={%Settings}(descr_row,2)
          Description(descr_row,3)="(from SETTINGS)"
     endif
Arguments(!a,1)=""
Description(descr_row,1)=%keyword
Description(descr_row,2)=%argument
EndSub 'Find_argument

'------------------------ Check a program argument   ------------------------
Subroutine Check_argument(string %argument, string %default, string %alt, table Description, scalar descr_row)
     if %argument="" then
          %argument=%default
          Description(descr_row,2)=%argument
          Description(descr_row,3)="(by default)" 
     endif
     if %argument<>%default and %argument<>%alt then
         Description(descr_row,3)="Invalid"
         %Message="Invalid argument (should be "+%default+"/"+%alt+"): "+Description(descr_row,1)+%argument 
         StatusLine %Message
         stop
     endif
EndSub 'Check

'--------- Check a layout data (number of row/column of data)  ---------
Subroutine Check(scalar RC, string %row_or_col, string %Address, string %Text, string %FileName)
     if %row_or_col="col" then
          !max=256
     else
          !max=65536 
     endif 
     if RC=NA or RC<1 or RC>!max or @round(RC)<>RC then
          Description(5,4)="Error"
          StatusLine Error in cell {%Address} of {%FileName} {%Text}
          stop
     endif
EndSub 'Check

'------------- Compute lettering of a column from its number -------------
Subroutine Column(scalar Column, string %Column)
%ABC="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
%Column=""
     !f=@floor(Column/26-0.03)
          if !f>0 then
               %Column=%Column+@mid(%ABC,!f,1)
          endif
!s=Column-!f*26
%Column=%Column+@mid(%ABC,!s,1)
EndSub 'Column

'---------------------- Find p-value for a given t-statistic ----------------------
Subroutine Find_p_value(scalar t_stat, matrix p_val, matrix CriticalValue, scalar P, scalar break_point, scalar p_value)
     for !p=1 to P
          if t_stat<=CriticalValue(!p,break_point) or p_val(!p,1)=1 then
               ExitLoop
          endif
     next
p_value=p_val(!p,1)
EndSub 'p_value

'-------------- Calculate the Phillips-Perron statistic Z(t-stat) --------------
Subroutine PP_statistic(series eps, scalar t_stat, scalar se_lambda, scalar se_resid, scalar Z, scalar Lags)
'       Input parameters:
'           eps is the regression residuals
'           t_stat is the t-ratio of lambda
'           se_lambda is the standard error of lambda
'           se_resid is the standard error of the regression
'                            (standard deviation of the residuals).
'       Output parameters:
'           Z is Z(t-stat), the Phillips-Perron test statistic
'           Lags is the bandwidth 
      
!_T=@obs(eps)                                            'sample size (t=0,...,!_T)

'Calculate residual variance without correction (!g0)
!g0=0                                                    
     for !t=1 to !_T
          !g0=!g0+eps(!t+1)*eps(!t+1)/!_T      '(as actually t=1,...,!_T+1)
     next

'Calculate HAC corrected residual variance (!f0)
!n=@floor(4*(!_T/100)^(2/9))                      'truncation lag
!s0=!g0
!s1=0
     for !j=1 to !n
          for !t=1+!j to !_T
               !s0=!s0+2*eps(!t+1)*eps(!t+1-!j)/!_T
               !s1=!s1+2*!j*eps(!t+1)*eps(!t+1-!j)/!_T
          next
     next
Lags=@floor(1.1447*((!T_-1)*(!s1/!s0)^2)^(1/3)) 
'                         (the Newey-West number of lags)
     if Lags>!_T-1 then
          Lags=!_T-1
     endif
!f0=0
     for !j=1 to Lags
          for !t=1+!j to !_T
               !f0=!f0+2*(1-!j/(Lags+1))*eps(!t+1)*eps(!t+1-!j)
          next
     next
!f0=!g0+!f0/!_T

'Calculate Phillips-Perron statistic
Z=t_stat*sqr(!g0/!f0)-!_T*(!f0-!g0)*se_lambda/(2*sqr(!f0)*se_resid)

EndSub 'PP_statistic

'****************** PREPARATORY PROCEDURES ******************
'---------------------- READ PROGRAM ARGUMENTS  ----------------------
!N_of_args=8
     if @IsObject("Arguments") then
          delete Arguments
     endif
     if @IsObject("Description") then
          delete Description
     endif
Table(!N_of_args+1,1) Arguments
Table(!N_of_args+15,4) Description
SetColWidth(Description, 1,7)
SetColWidth(Description, 2,20)
Description(1,1)="ARGUMENTS"
Description(1,3)=%0+" "+%1+" "+%2+" "+%3+" "+%4+" "+%5+" "+%6+" "+%7+" "+%8+" "+%9+" "+%10
     if %8<>"" then
          StatusLine Superfluous argument(s): ... {%8} {%9} {%10}
          stop
     endif
Arguments(1,1)=%0
Arguments(2,1)=%1
Arguments(3,1)=%2
Arguments(4,1)=%3
Arguments(5,1)=%4
Arguments(6,1)=%5
Arguments(7,1)=%6
Arguments(8,1)=%7
Arguments(9,1)=""
Show Description

Call Find_argument(Arguments, !N_of_args, "series=", 7, %Series, Description, 2, "Settings")
     if %Series="" then
          %Message="No series name specified: "+Description(1,3)
          StatusLine {%Message}
          stop
     endif
!errors=@errorcount
!T_=@obs({%series})
     if !errors<>@errorcount then
          Description(2,3)="Non-existent"
          StatusLine Workfile does not contain series {%Series}
          stop                    
     endif

Call Find_argument(Arguments, !N_of_args, "type=", 5, %type, Description, 3, "Settings")
!Reversed=(@left(%type,1)="-")
     if !Reversed then
          %type=@mid(%type,2) 
     endif
Call Check_argument(%type, "c", "nc", Description, 3)
    if %type="nc" then
         %type="none"
         %const=""
    else
         %type="const"
         %const="+c(2)"
    endif

Call Find_argument(Arguments, !N_of_args, "break=", 6, %break, Description, 4, "Settings")
!i=1
     while @mid(%break, !i,1)<>"-" and @mid(%break, !i,1)<>""
           !i=!i+1
     wend
%break_from=@left(%break,!i-1)
%break_to=@mid(%break, !i+1)
!i=1
     while @mid(%break_to, !i,1)<>";" and @mid(%break_to, !i,1)<>""
           !i=!i+1
     wend
%step=@mid(%break_to, !i+1)
     if %step="" then
          %step="1"
     endif
%break_to=@left(%break_to,!i-1)
!step=@val(%step)
     if !step=NA or !step<=0 or @round(!step)<>!step then
         Description(4,3)="Invalid step" 
         StatusLine Invalid step: break={%break} 
         stop
     endif

Call Find_argument(Arguments, !N_of_args, "CVfile=", 7, %CVfile, Description, 5, "Settings")
     if %CVfile="" then
          %CVfile="CriticalValues.xls"
          Description(5,2)=%CVfile
          Description(5,3)="(by default)" 
     endif

Call Find_argument(Arguments, !N_of_args, "T=", 2, %Tcv, Description, 6, "Settings") 
     if %Tcv="" then
          !Tcv=!T_-1
          Description(6,2)=@str(!Tcv)
          Description(6,3)="(by default)"
     else
          !Tcv=@val(%Tcv)
               if !Tcv=NA or !Tcv<2 or @round(!Tcv)<>!Tcv then
                    Description(6,3)="Invalid"
                    StatusLine Invalid samle size for testing: T={%Tcv}
                    stop
               endif
     endif
           
Call Find_argument(Arguments, !N_of_args, "Perron=", 7, %Perron, Description, 7, "Settings")
     if @left(%Perron,3)<>"yes" or @left(%Perron,3)="" then
          Call Check_argument(%Perron, "no", "yes", Description, 7)
     else
               if %Perron="yes" then
                    %Perron="yes,pt"
               endif 
          Call Check_argument(%Perron, "yes,pt", "yes,or", Description, 7)
     endif
Description(7,2)=%Perron
%Perron_version=@right(%Perron,2)
%Perron=@left(%Perron,3)

Call Find_argument(Arguments, !N_of_args, "DF=", 3, %DF, Description, 8, "Settings")
Call Check_argument(%DF, "yes", "no", Description, 8)

Call Find_argument(Arguments, !N_of_args, "keep=", 5, %keep, Description, 9, "Settings")
Call Check_argument(%keep, "yes", "no", Description, 9)

     for !i=1 to !N_of_args
          if Arguments(!i,1)<>"" then
               Description(10,1)=Arguments(!i,1) 
               Description(10,3)="Unidentified"
               %Message="Unidentified argument: "+Description(10,1)
               StatusLine  %Message
               stop
          endif
     next
SetLine(Description,10)      
Delete Arguments

'-------------------- Read critical values of the test(s) --------------------
Matrix(6,1) Layout

'***** Read map of the file 
!errors=@errorcount
Layout.read(rect,s=Map,B2)  {%CVfile}
     if !errors<>@errorcount then
          Description(5,4)="Error"
          StatusLine Error while reading {%CVfile} or it does not contain a sheet named Map
          stop
     endif
!p_value_col=Layout(1,1)
Call Check(!p_value_col, "col", "Map!B2", "(column of p-values)", %CVfile)

!Break_points_row=Layout(2,1)
Call Check(!Break_points_row, "row", "Map!B3", "(row of break points)", %CVfile)

!No_of_sampe_sizes=Layout(3,1)
Call Check(!No_of_sampe_sizes, "col", "Map!B4", "(the number of samle sizes)", %CVfile)

!Data_beginning=Layout(4,1)
Call Check(!Data_beginning, "row", "Map!B5", "(first row of the data)", %CVfile)
          
!Data_end=Layout(5,1)
Call Check(!Data_end, "row", "Map!B6", "(last row of the data)", %CVfile)
     if !Data_end<!Data_beginning then
          Description(5,4)="Error"
          StatusLine The numer of the first row of the data is smaller than that of the last row (cells Map!B5 and B6 of {%CVfile})
          stop
     endif
!P_=!Data_end-!Data_beginning+1
          
!Allocation_table_row=Layout(6,1)
Call Check(!Allocation_table_row, "row", "Map!B7", "(first row of the allocation table)", %CVfile)

Delete Layout

'***** Find row in the data allocation table for a given sample size
Matrix(!No_of_sampe_sizes+1,1) Sample_size
Sample_size.read(rect,s=Map,A{!Allocation_table_row})  {%CVfile}
Sample_size(!No_of_sampe_sizes+1,1)=NA
     for !r=1 to !No_of_sampe_sizes
          if !Tcv<=Sample_size(!r,1) then
                    ExitLoop
          endif      
     next
!high=Sample_size(!r,1)
!low=NA          
     if !r>1 then 
          !low=Sample_size(!r-1,1)
     endif 
     if !Tcv<>Sample_size(!r,1) or !r>!No_of_sampe_sizes then
          Description(6,4)="NA"
          StatusLine No statistics for T={!Tcv}. The nearest values are  {!low} and {!high}.
          stop
     endif
!r=!Allocation_table_row+!r-1
%r=@str(!r)           

Delete Sample_size

'***** Read data allocation table
Matrix(1,12) CVlayout
CVlayout.read(rect,s=Map,B{!r}) {%CVfile}
Vector(2) Start
Vector(2) End
Vector(2) N_of_breaks
Vector(2) Do=1
Do(2)=(%Perron="yes")
Table(2,1) sheet
sheet(1,1)="Nonlin. tau-"
sheet(2,1)="Perron tau-"
     if %type="none" then
          Start(1)=CVLayout(1,1)
          End(1)=CVLayout(1,2)       
          Start(2)=CVLayout(1,7)
          End(2)=CVLayout(1,8)
          %ATcolumn="BCHI"
          sheet(1,1)=sheet(1,1)+"0"
          sheet(2,1)=sheet(2,1)+"0"
     endif
     if !reversed and %type="none" then
          Start(1)=CVLayout(1,3)         
          End(1)=CVLayout(1,4)       
          Start(2)=CVLayout(1,9)
          End(2)=CVLayout(1,10)
          %ATcolumn="DEJK"
          sheet(1,1)=sheet(1,1)+"(r)"
          sheet(2,1)=sheet(2,1)+"(r)"
     endif
     if %type="const" then
          Start(1)=CVLayout(1,5)
          End(1)=CVLayout(1,6)       
          Start(2)=CVLayout(1,11)
          End(2)=CVLayout(1,12)
          %ATcolumn="FGLM"
          sheet(1,1)=sheet(1,1)+"mu"
          sheet(2,1)=sheet(2,1)+"mu"
     endif
Delete CVlayout

     for !i=1 to 2
          N_of_breaks(!i)=End(!i)-Start(!i)+1
               if Do(!i) and (Start(!i)=NA or Start(!i)<1 or Start(!i)>256 or @round(Start(!i))<>Start(!i) or End(!i)=NA or End(!i)<1 or End(!i)>256 or @round(End(!i))<>End(!i) or N_of_breaks(!i)<0) then
                    %col_f=@mid(%ATcolumn, !i*2-1,1)
                    %col_l=@mid(%ATcolumn, !i*2,1)
                    Description(5,4)="Error"
                    Delete Start End Do N_of_breaks
                    %sheet=sheet(!i,1)
                    StatusLine No {%sheet} statistic for T=!Tcv or error in cell Map!{%col_f}{%r} or {%col_l}{%r} of {%CVfile}
                    stop
               endif
     next
     if %Perron="yes" and N_of_breaks(1)<>N_of_breaks(2) then
          Description(7,4)="Blocked: Break points differ from those in "+%sheet1
          Do(2)=0
          %Perron="no"
     endif
!N_of_breaks=N_of_breaks(1)
Delete N_of_breaks

'***** Read test statistics
'Read p-values
     for !i=1 to 2
          if Do(!i) then
               Matrix(!P_,1) p_value{!i}
               Call Column(!p_value_col,%Column)
               !errors=@errorcount
               %sheet=sheet(!i,1)
               p_value{!i}.read(rect,s={%sheet},{%Column}{!Data_beginning})  {%CVfile}
                    if !errors<>@errorcount then
                         Description(5,4)="Error"
                         Delete Start End Do p_value*
                         StatusLine File {%CVfile} does not contain a sheet named %sheet{!i}
                         stop
                    endif
          endif
     next
                    
'Read break points and critical values
     for !i=1 to 2
          if Do(!i) then
               Matrix(1,!N_of_breaks) BreakPoints{!i}
               Matrix(!P_,!N_of_breaks) CriticalValues{!i}
               Call Column(Start(!i),%Column)
               %sheet=sheet(!i,1)
               BreakPoints{!i}.read(rect,s={%sheet},{%Column}{!Break_points_row})  {%CVfile}
               CriticalValues{!i}.read(rect,s={%sheet},{%Column}{!Data_beginning})  {%CVfile}
          endif
     next
     if %Perron="yes" then
     if BreakPoints1<>BreakPoints2 then
          Description(7,4)="Blocked: Break points differ from those in "+%sheet1
          %Perron="no"
          Delete CriticalValue2 p_value2
     endif
     endif
Matrix Break_Points=@transpose(BreakPoints1)
Delete Start End Do sheet BreakPoints*              

'***** Arrange break points
'Defaults if not specified
     if %break_from="" then               
           %break_from=@OtoD(@round(Break_Points(1,1)*(!T_-1))+2)
           !break_to=@round(Break_Points(!N_of_breaks,1)*(!T_-1))+2
                if !break_to>!T_-1 then
                     !break_to=!T_-1
                endif
           %break_to=@OtoD(!break_to)
           Description(4,2)=%break_from+"-"+%break_to+";1"
           Description (4,3)="(by default)"    
     endif
     if %break_to="" then
          %break_to=%break_from
     endif

'Check specification of break points
!errors=@errorcount
!break_from=@DtoO(%break_from)
     if !break_from<2 or !break_from>!T_-1 then
          !errors=!errors+1
     endif
!break_to=@DtoO(%break_to)
     if !break_to<1 or !break_to>!T_-1 or !break_to<!break_from then
          !errors=!errors+1
     endif
     if !errors<>@errorcount then
         Description(4,3)="Invalid" 
         StatusLine Invalid break date(s): break={%break} 
         stop
     endif
     if %break_from<>"" and %break_to="" then
          %break_to=%break_from
     endif
!i=(!break_to-!break_from)/!step+1
!j=@round(!i)
     if @abs(!i-!j)>10e-7 then
          Description(4,3)="Invalid step or dates: "+@str(!i)+" break points"
          StatusLine Noninteger number of breaks: ({%break_to} - {%break_from})/!step + 1 = !i
          stop
     endif

'Match break points with those available in the statistic tables 
Series StatCol=NA
Description(11,1)="BREAK POINTS:"
Description(12,1)="SAMPLE: t=0,...,"+@str(!T_-1)+"; TEST STATISTICS FOR: t=0,...,"+@str(!Tcv)
     for !i=13 to 17 step 2
          Description(!i,2)="specified"
          Description(!i+1,2)="test statistics for"
     next
Description(13,1)="Date:"
Description(15,1)="Theta:"
Description(17,1)="Theta/T:"
!i=3
     for !b=!break_from to !break_to step !step
          Description(13,!i)=@OtoD(!b)
          SetCell(Description,15,!i,!b-2,0)
          !THETA=(!b-2)/(!T_-1)
          SetCell(Description,17,!i,!THETA,3)
          !min=1
               for !j=1 to !N_of_breaks
                    if @abs(!THETA-Break_Points(!j,1))<!min then
                         !min=@abs(!THETA-Break_Points(!j,1))
                         !m=!j
                    endif
               next
          StatCol(!b)=!m
          SetCell(Description,18,!i,Break_Points(!m,1),3)
          !br=@round(Break_Points(!m,1)*(!T_-1))
          SetCell(Description,16,!i,!br,0)
          Description(14,!i)=@OtoD(!br+2)
          !i=!i+1
     next 

Delete  Break*
Table(9,2) Settings
     for !i=2 to 9
          Settings(!i,1)=Description(!i,1)
          Settings(!i,2)=Description(!i,2)
     next
Settings(1,1)="SETTINGS:"

'****************** ESTIMATE AND TEST MODELS *******************
Equation Perron_type
Equation Original_Perron
Equation Nonlinear
Vector c0=c
c0=0
Vector cP=c
Vector c_1=c
Vector cb=c
Vector c_best=c
c_1=0
     if @IsObject("Dickey_Fuller_test") then
          Delete Dickey_Fuller_test Phillips_Perron_test
     endif
     if %DF="yes" then
          freeze(Dickey_Fuller_test) {%series}.uroot(adf,{%type},dif=0,lag=0)
          Show Dickey_Fuller_test
          freeze(Phillips_Perron_test) {%series}.uroot(pp,{%type},dif=0,hac=bt,band=nw)
          Show Phillips_Perron_test
     endif
     
!minSSR_nl=1000000
!minSSR_P=1000000
    for !b=!break_from to !break_to step !step
         Series Break{!b}=0
               for !t=!b to !T_
                     Break{!b}(!t)=1
               next
               if !reversed then
                     Break{!b}=1-Break{!b}
               endif

     'Perron test
         Perron_type.ls D({%series})=c(1)*{%series}(-1){%const}+c(3)*Break{!b}+c(4)*Break{!b}(-1)
               if @SSR<!minSSR_P then
                    !minSSR_P=@SSR
                    !best_P=!b
               endif
          cP=c

     'Nonlinear test
          !minSSR=1000000
               for %c P _1 0
                    c=c{%c}
                    Nonlinear.ls(M=1000,C=1e-07)  D({%series})=c(1)*{%series}(-1){%const}+c(3)*(Break{!b}-(c(1)+1)*Break{!b}(-1))
                        if @SSR<!minSSR then
                              !minSSR=@SSR
                              cb=c
                        endif
               next '%c
               if !minSSR<!minSSR_nl then
                       !minSSR_nl=!minSSR
                       c_best=cb
                       c_1=cb
                       !best_nl=!b
               endif
     next '!b

Delete c0 c_1 cP cb 

'*************************** OUTPUT ****************************
!p_value=0
!Z=0
!lags=0
     if @IsObject("UR_tests") then
          Delete UR_tests
     endif
Table(12,5) UR_tests
UR_tests(1,1)="TEST ON "+%series+" FOR A UNIT ROOT SUBJECT TO BREAK"
SetLine(UR_tests,2)
UR_tests(3,2)="Nonlinear model"
UR_tests(4,1)="Break at:"
UR_tests(4,2)=@OtoD(!best_nl)
UR_tests(5,2)="statistic"
UR_tests(5,3)="p-value"
UR_tests(6,1)="t-stat"
UR_tests(7,1)="Z(t-stat)"
UR_tests(8,1)="Bandwidth"
c=c_best
Nonlinear.ls(M=1000,C=1e-07)  D({%series})=c(1)*{%series}(-1){%const}+c(3)*(Break{!best_nl}-(c(1)+1)*Break{!best_nl}(-1))
!t_stat=@tstats(1)
Call PP_statistic(resid, !t_stat, @stderrs(1), @se, !Z, !lags)
SetCell(UR_tests,6,2,!t_stat,3)
Call Find_p_value(!t_stat, p_value1, CriticalValues1, !P_, StatCol(!best_nl),!p_value)
SetCell(UR_tests,6,3,!p_value,3)
SetCell(UR_tests,7,2,!Z,3)
Call Find_p_value(!Z, p_value1, CriticalValues1, !P_, StatCol(!best_nl),!p_value)
SetCell(UR_tests,7,3,!p_value,3)
SetCell(UR_tests,8,2,!lags,0)

     if %Perron="yes" then
          UR_tests(3,4)="Perron-type model"
          UR_tests(4,4)=@OtoD(!best_P)
          UR_tests(5,4)="statistic"
          UR_tests(5,5)="p-value"
               if %Perron_version="pt" then
                    Perron_type.ls D({%series})=c(1)*{%series}(-1){%const}+c(3)*Break{!best_P}+c(4)*Break{!best_P}(-1)
                         if @IsObject("Original_Perron") then
                              Delete Original_Perron
                         endif
                    Perron_type.results
               else
                    Original_Perron.ls D({%series})=c(1)*{%series}(-1){%const}+c(5)*Break{!best_P}+c(4)*D(Break{!best_P})
                         if @IsObject("Perron_type") then
                              Delete Perron_type
                         endif
                    Original_Perron.results
               endif
          !t_stat=@tstats(1)
          Call PP_statistic(resid, !t_stat, @stderrs(1), @se, !Z, !lags)
          SetCell(UR_tests,6,4,!t_stat,3)
          Call Find_p_value(!t_stat, p_value2, CriticalValues2, !P_, StatCol(!best_P),!p_value)
          SetCell(UR_tests,6,5,!p_value,3)
          SetCell(UR_tests,7,4,!Z,3)
          Call Find_p_value(!Z, p_value2, CriticalValues2, !P_, StatCol(!best_P),!p_value)
          SetCell(UR_tests,7,5,!p_value,3)
          SetCell(UR_tests,8,4,!lags,0)
     else
          if @IsObject("Perron_type") then
               Delete Perron_type
          endif
          if @IsObject("Original_Perron") then
               Delete Original_Perron
          endif
     endif
SetLine(UR_tests,9)
UR_tests(10,1)="''t-stat' is the t-ratio of C(1) (lambda)."
UR_tests(11,1)="''Z(t-stat)' is its Pillips transformation, bandwidth beeing the number of lags used." 
UR_tests(12,1)="In the regression outputs, C(1) stands for lambda (autoregression coefficient - 1),"
UR_tests(13,1)="                                      C(2) stands for alfa (constant),"
UR_tests(14,1)="                                      C(3) stands for gamma (break height),"
UR_tests(15,1)="                                      C(4) stands for delta (in both versions of Perron''s regression),"
UR_tests(16,1)="                                and C(5) stands for psi (in the original Perron''s regression)."
Nonlinear.results
Show UR_tests

Delete c_best CriticalValues* p_value* StatCol

     if %keep="no" then                                                    'Clean user's workfile
          Delete Break* Settings UR_tests Nonlinear Description
               if @IsObject("Perron_type") then
                    Delete Perron_type
               endif
               if @IsObject("Original_Perron") then
                    Delete Original_Perron
               endif
               if @IsObject("UR_tests") then
                    Delete UR_tests
               endif
               if @IsObject("Dickey_Fuller_test") then
                    Delete Dickey_Fuller_test Phillips_Perron_test
               endif
     endif

'END OF PROGRAM

