'This program estimates the cumulative distribution functions
'         (critical values) of the unit root test statistic (t-ratio of lambda)
'         for the nonlinear and Perron tests allowing for break,
'         as well as those for the Dickey-Fuller tests,  without and with
'         constant, across simulated random walks with
'         innovations ~ N(0, 1).
'         See Instructions.pdf for details.

'         Version: 08.2005

'         Author: Konstantin Gluschenko
'                       glu@nsu.ru
'                       http://econom.nsu.ru/users/gluschenko

'ARGUMENTS of the program may follow in any order, but all 5 of them should be provided except for the case when the workfile already exists on disc (e.g., estimations are processing in parts). Only the filename should be provided in such a case (then keyword "file=" may be omitted).

'The arguments are as follows: 
'(<...> means some value of argument, [...] means an optional part of argument, | means "or", and "and so on" means replication of previous expression):

' file=[<path>]<file_name> is the workfile name 
'          (if there are blanks, enclose it in quotes).
' T=<number> is the sample size ( t=0,...,T). 
'          (In the workfile and program, it will be t=1,...,T+1.)
' N=<number> is the number of  replications.
' type=nc|c is the type of equations to be estimated:
'          nc means AR(1) without constant;
'          c    means AR(1) with constant.
' breaks=[-]<numberA>,...,<numberB>;<numberC>
'                |[-]<number1>[,[-]<number2>][,[-]<number3>[,and so on]]]
'          is the list of break points. 
'          The upper form specifies them as a sequence from 
'          <numberA> through <numberB> with increment <numberC>; 
'          the lower  one lists any number of specific points.
'          The break points may be either relative (as fractions of  the 
'          sample  size) or absolute (as specific values of t).  However,
'          these types should not be mixed in the same list. 
'          Minus means a break(s) to be "reversed", that is, 
'          the break dummy=1if t<=the break point, and 0 otherwise.
'          In the upper form of the argument, "-" concerns all break 
'          points; in the lower one, it concerns the given break point.  
'          (Note that <numberC> may also be negative;  this implies 
'          just a negative increment -- then  <numberA> should be
'          greater than <numberB>.)

'EXAMPLES: 
'  file="test 1" type=nc breaks=-75,25  N=100 T=100
'  file=test  type=c  N=200000 T=50  breaks=0.1,...,0.9;0.1
'  file="50 nc, reversed"  type=nc  N=200000 T=50  breaks=-0.9,...,0.1;-0.1
'  type=nc breaks=70,-25,-28,-29,-30,-31,-32,-35  file=BAK N=1000 T=100

' Before starting the program, set "Maximum errors before halting"  
'                                                                       to a number more than 1 ! 

' If workfile already exists on disk, the program cannot change its
'                                parameters (i.e., samle size, equation type, etc.).

'RESULTS:
'CDF_t_nl, CDF_t_P, CDF_DF_nc, and CDF_DF_c are CDFs of t-ratio of the autoregressive coefficient (minus one) in nonlinear test equation, Perron-type equation, and Dickey-Fuller's CDF for their equations without and with constant, respectively. The first column is probability (from 0 to 1 with step 0.001); the rest are critical values by break point. (Thus, the number of column in CDF_t_nl and CDF_t_P is the number of break point + 1.)
'Matrices t_nl, t_P, t_DF_nc, and t_DF_c contain the full set of estimates of the t-ratio of lambda.
'Table DESCRIPTION reports conditions of a given estimation. 

'*************************** SUBROUTINES ***************************
'------------ Find an argument in the program argument list  -------------
Subroutine Find_argument(string %Source, string %List, string %List_length, string %keyword, string %kw_length, string %name, string %argument, string %arg_length)
!L=@val(%List_length)
!length=@val(%kw_length)
!first=1
     while !first<!L
              for !last=!first+1 to !L
                    if @mid(%List, !last,1)="#" or %List="" then
                    ExitLoop
                    endif
               next
               if @mid(%List,!first,!length)=%keyword then
                    %arg_length=@str(!last-!first-!length)
                    %argument=@mid(%List,!first+!length,!last-!first-!length)
                    %List=@left(%List,!first-1) + @mid(%List,!last+1)
                    %List_length=@str(!L-(!last-!first)-1)
                    ExitLoop
               else
                    !first=!last+1
               endif
     wend
     if %argument="" then
          StatusLine No {%name} specified: {%Source}
          stop
     endif
EndSub 'Find_argument

'-------------- Transform seconds to hours and minutes ------------------
Subroutine Seconds_to_HM(scalar Seconds, string %Time)
     !minutes=@round(Seconds/60)
     !hours=@floor(!minutes/60)
     !minutes=!minutes-!hours*60
     %Time=""
          if !hours>0 then
               %Time=@str(!hours)+" hours "
          endif
    %Time=%Time+@str(!minutes)+" min. "
EndSub 'Seconds_to_HM

'------------ Computing CDFs of a given set of t-ratio series -------------
Subroutine CDF(matrix Tau, scalar Rows, scalar Cols, matrix CDF, scalar Quantiles)
Series X
     for !i=0 to Quantiles
        CDF(!i+1,1)=!i/Quantiles
     next
!step=Rows/Quantiles
     for !j=1 to Cols
              for !t=1 to Rows
                   X(!t)=Tau(!t,!j)
              next
         Sort X         
         !i=@floor(!step/2-1)
               if !i>0 then
                    CDF(1,!j+1)=X(!i)
                    CDF(Quantiles+1,!j+1)=X(Rows-!i)
               else
                    CDF(1,!j+1)=X(1)-1e-6
                    CDF(Quantiles+1,!j+1)=X(Rows)+1e-6
               endif
               for !i=1 to Quantiles-1
                    CDF(!i+1,!j+1)=X(@ceiling(!i*!step))
              next

     next
delete X
EndSub 'CDF

'****************** PREPARATORY PROCEDURES ******************

'--- READ PROGRAM ARGUMENTS AND FIND WORKFILE NAME ---
tic
%Arguments=%0+"#"+%1+"#"+%2+"#"+%3+"#"+%4+"#"
%Source=%0+" "+%1+" "+%2+" "+%3+" "+%4
     if %5<>"" then
          StatusLine Superfluous argument(s): ...{%5} {%6} {%7} {%8}
          stop
     endif
     if %1="" and @left(%Arguments,5)<>"file=" then
          %Arguments="file="+%Arguments
     endif
     for !i=1 to 4096
            if @mid(%Arguments, !i,1)="" then
            ExitLoop
            endif
     next
%List_length=@str(!i-1)
Call Find_argument(%Source, %Arguments, %List_length, "file=", "5", "workfile name", %file, %a)
!errors=@errorcount
Load {%file}

'---------------------- WORKFILE %0 EXISTS ON DISK. ----------------------
'                                THEN READ ITS PARAMETERS

     if !errors=@errorcount then 'old file
                if %1<>"" then
                    StatusLine The file already exists on disc. Then other arguments are superfluous: {%Source}
                    stop                    
               endif
          !T_=T+1                                                     'Sample size
          !N=N                                                           'Number of replications
               if Description(5,2)<>"" then
                    SetCell(Description,5,2,@str(Current),"r")
               else
                    StatusLine The estimation has been completed.
                    stop
               endif
     else 'new file 

'---------------------- WORKFILE %0 DOES NOT EXIST. ---------------------
'          THEN CHECK PROGRAM ARGUMENTS, CREATE NEW       
'              WORKFILE %0,  AND CREATE BREAK VARIABLES

'***** Check arguments
          Call Find_argument(%Source, %Arguments, %List_length, "T=", "2", "sample size", %T,%a) 
           !T_=@val(%T)
                    if !T_=NA or !T_<5 or @round(!T_)<>!T_ then 
                         StatusLine Invalid samle size: T=%T
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "N=", "2", "number of replications", %N,%a)
           !N=@val(%N)
                    if !N=NA or !N<1 or @round(!N)<>!N then 
                         StatusLine Invalid number of replications: N=%N
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "type=", "5", "type", %type, %a)
                     if %type<>"nc" and %type<>"c" then 
                         StatusLine Invalid type (should be c or nc): type=%type 
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "breaks=", "7", "break points", %breaks, %a)
          !Nb=1
          !Number_of_breaks=NA
          !reversed=NA
          !l=@val(%a)
          !j=0
               for !i=2 to !l
                    if @mid(%breaks,!i,5)=",...," then
                         !j=!i
                    endif
                    if @mid(%breaks,!i,1)=";" then
                         ExitLoop
                    endif
               next
               if !j<!l-6 and !j>1 and !i<!l then                     'breaks=b1,...,bn;d
                    !reversed=(@left(%breaks,1)="-")
                    !from=abs(@val(@left(%breaks,!j-1)))
                    !to=@val(@mid(%breaks,!j+5,!i-(!j+5)))
                    !by=@val(@mid(%breaks,!i+1))
                    !Nb=(!to-!from)/!by+1+10e-8
                       if  !from=NA or !to=NA or !by=NA or !Nb<2 or !from<=0 or !to<=0 or (@abs(!by)<1 and (!from>=1 or !to>=1)) or (@abs(!by)>=1 and (!from<1 or !to<1 or @round(!by)<>!by)) then
                           StatusLine Invalid break argument: breaks=%breaks
                           stop
                       endif 
                    !Number_of_breaks=@round(!Nb)
                       if @abs(!Number_of_breaks-!Nb)>10e-7 then
                           StatusLine Noninteger number of breaks: (!to-!from)/!by+1=!Nb
                           stop
                       endif  
                    !break=!from-!by 
               endif  'breaks=b1,...,bn;d
               if !Number_of_breaks=NA then
                    !Number_of_breaks=1
                         for !i=1 to !l
                               if @mid(%breaks,!i,1)="," then
                               !Number_of_breaks=!Number_of_breaks+1
                               endif
                         next 
               endif

'***** Create new workfile
          !T_=!T_+1
          WorkFile {%file} u 1 !T_
          Scalar T=!T_-1
          Scalar N=!N
          Scalar Number_of_breaks=!Number_of_breaks
          Scalar Seconds=0
          Scalar Current=1

          Table(11+!Number_of_breaks,5) Description
          SetColWidth(Description,1,4)
          SetColWidth(Description,2,15)
          Description(1,1)=%type
               if %type="nc" then 
                    Description(1,2)="Regressions without intercept term"
               else
                    Description(1,2)="Regressions with intercept term"
               endif
          Description(3,1)="Sample size: "+%T
          Description(4,1)="Number of replications: "+%N
          Description(6,1)=@str(Number_of_breaks)
          SetCell(Description,5,2,"0","r")
          Description(5,3)="or more performed (see scalar CURRENT for exact figure) "
          SetCell(Description,6,2,"BREAKS:","l")
          Description(7,1)="No."
          Description(7,2)="Break point"
          Description (7,3)="Note"
          Description(9+Number_of_breaks,1)="Started "+@date+" at "+@time
          DEscription(12+Number_of_breaks,1)=%Source 
          Vector(Number_of_breaks) abs_point
          Vector(Number_of_breaks) rel_point
          Vector(Number_of_breaks) reversed=!reversed
          !i=0
               for !b=1 to Number_of_breaks
                    if !Nb>1 then
                         !break=!break+!by
                         !rel=(!break<1)
                    else
                              for !j=!i+1 to !l
                                   if @mid(%breaks,!j,1)="," then
                                   ExitLoop
                                   endif
                              next
                         reversed(!b)=(@mid(%breaks,!i+1,1)="-")
                         !break=abs(@val(@mid(%breaks,!i+1,!j-!i-1)))
                         !i=!j
                              if !b=1then
                                   !rel=(!break<1)
                              endif
                    endif
                    if !break=NA or !break<=0 or !break>T-1 or !rel<>(!break<1) then
                         close {%file}
                          StatusLine Invalid break argument: breaks=%breaks
                         stop
                    endif
                    if !break<1then
                         abs_point(!b)=@round(!break*T)
                         rel_point(!b)=!break
                    else
                         abs_point(!b)=!break
                         rel_point(!b)=!break/T
                    endif
             next '!b
StatusLine %Source

'***** Create break variables (dummies)               
               for !b=1 to Number_of_breaks
                     Series Break{!b}=0
                             for !t=abs_point(!b)+2 to !T_
                                  Break{!b}(!t)=1
                             next
                      Description(7+!b,1)=@str(!b)
                      Description(7+!b,2)=@str(abs_point(!b))+ " ("+@str(rel_point(!b))+"*T)"
                          if reversed(!b) then
                               Break{!b}=1-Break{!b}
                               Description(7+!b,3)="reversed break (111...000)"
                          endif
               next
          Delete abs_point rel_point reversed
     endif  'new file

'************************ ESTIMATE MODELS *************************
Series Y=0
Equation ARnc
Equation ARconst
Equation ARnl_break
Equation ARP_break

'Estimated t-stats: "NL" for nonlinear, "P" for Perron-type equation
Matrix(N,Number_of_breaks) t_NL 
Matrix(N,Number_of_breaks) t_P
Matrix(N,1) t_DF_nc
Matrix(N,1) t_DF_c
Series eps
   
     if Description(1,1)="nc" then                      'without/with const
         %const=""          
     else
         %const="+c(2)"
     endif
Show Description
!Backup_time=3600 'seconds, i.e. 1 hour
!Last_backup=@toc
!Elapsed_time=Seconds+@toc
!k1=Current

    for !k=!k1 to N
         Call Seconds_to_HM((Seconds/!k)*(N-!k),%a)
         StatusLine Replication {!k} of {!N}. Estimated time left {%a}
         Current=!k
         
     'Generate a random walk
         eps=nrnd
         smpl @first+1 @last
         Y=Y(-1)+eps
         smpl @first @last

     'Dickey-Fuller test equations
         ARnc.ls D(Y)=c(1)*Y(-1)
         t_DF_nc(!k, 1)=ARnc.@tstats(1)
         ARconst.ls D(Y)=c(1)*Y(-1)+c(3)
         t_DF_c(!k, 1)=ARconst.@tstats(1)

              for !b=1 to Number_of_breaks
                     'Perron-type specification
                     ARP_break.ls D(Y)=c(1)*Y(-1){%const}+c(3)*Break{!b}+c(4)*Break{!b}(-1)
                     t_P(!k, !b)=ARP_break.@tstats(1)

                     'Nonlinear specification
                      c=0
                      ARnl_break.ls(M=1000,C=1e-07) D(Y)=c(1)*Y(-1){%const}+c(3)*(Break{!b}-(c(1)+1)*Break{!b}(-1))
                      t_NL(!k, !b)=ARnl_break.@tstats(1)
              next '!b

         !toc=@toc
         Seconds=!Elapsed_time+!toc
               if !toc-!Last_backup>=!Backup_time then
                    SetCell(Description,5,2,@str(Current),"r")
                    save {%file}
                    Show Description
                    !Last_backup=!toc
               endif
    next '!k
Current=Current+1
save {%file}

delete Y eps AR*  Break*
range 1 !N
smpl @all  
!q=1000 'the number of quantiles
matrix(!q+1,2) CDF_DF_nc
StatusLine Computing Dickey-Fuller CDF (noconst)
call CDF(t_DF_nc, N, 1, CDF_DF_nc, !q)
matrix(!q+1,2) CDF_DF_c
StatusLine Computing Dickey-Fuller CDF (const)
call CDF(t_DF_c, N, 1, CDF_DF_c, !q)
matrix(!q+1,Number_of_breaks+1) CDF_t_NL
StatusLine Computing CDF for nonlinear test
call CDF(t_NL, N, Number_of_breaks, CDF_t_NL, !q)
matrix(!q+1,Number_of_breaks+1) CDF_t_P
StatusLine Computing CDF for Perron-type test
call CDF(t_P, N, Number_of_breaks, CDF_t_P, !q)
range 1 !T_
smpl @all

Description(5,2)=""
Description(5,3)=""
Description(10+Number_of_breaks,1)="Finished "+@date+" at "+@time
Seconds=!Elapsed_time+@toc
Call Seconds_to_HM(Seconds,%a)
Description(11+Number_of_breaks,1)="Elapsed time: "+%a
delete Current Seconds T N Number_of_breaks
save {%file}    
StatusLine Finished: N={!N}, %type. Elapsed time {%a}

'END OF PROGRAM

