'This program estimates the cumulative distribution functions
'         (critical values) of the unit root test statistic (t-ratio of lambda)
'         for the nonlinear and Perron tests allowing for break,
'         as well as those for the Dickey-Fuller tests,  without and with
'         constant, across simulated AR(1) series with a break; 
'         innovations ~ N(0, sigma^2).
'In contrast to program "CDFs of UR statistics with break.prg",
'         this program estimates CDFs across various initial 
'         conditions y(0) (with respect to the maginitude or the 
'         variance), or break heights ( gamma), or autoregression 
'         parameters (lambda), or constant, or s.d. of the innovations
'         in one run.  Since nonzero lambda(s) can be specified, 
'         this program can estimate CDFs not only for the unit root 
'         case, but for alternatives as well.
'         See Instructions.pdf for details.

'         Version: 08.2005

'         Author: Konstantin Gluschenko
'                       glu@nsu.ru
'                       http://econom.nsu.ru/users/gluschenko

'ARGUMENTS of the program may follow in any order.Only 3 of all the 11arguments are optional, having default values, namely "y0", var(y0)", and "sigma". The rest 8 arguments must be specified except for a case when the workfile already exists on disc (e.g., estimations are processing in parts). The filename only should be provided in such a case (then keyword "file=" may be ommited). 

'The arguments are as follows 
'(<...> means some value of argument, [...] means an optional part of an argument, | means "or", and "and so on" means replication of a previous expression):

' file=[<path>]<file_name> is the workfile name 
'          (if there are blanks, enclose it in quotes).
' T=<number> is the sample size ( t=0,...,T). 
'          (In the workfile and program, it will be t=1,...,T+1.)
' N=<number> is the number of  replications.
' type=nc|c is the type of equations to be estimated:
'          nc means regressions with no intercept;
'          c    means regressions with intercept.
' break=[-]<number1>[,[-]<number2>[,[-]<number3>[,and so on]]]
'          is the break point(s) as a fraction(s) of the sample size;
'          minus means a break to be "reversed", that is, the 
'          respective break dummy=1 if t<=the break point  and 0 
'          otherwise.  
' y0=<number1>[,<number2>[,<number3>[,and so on]]]
'          is the mean(s) of the initial value, y(0). By default, y0=0.
' var(y0)=<number1>[,<number2>[,<number3>[,and so on]]]
'         is the variance(s) of y0 (zero value implies that y0 is not 
'         random). By default, var(y0)=0.        
' gamma=<number1>[,<number2>[,<number3>[,and so on]]]
'         is the value(s) of the break height, gamma.
' const=<number1>[,<number2>[,<number3>[,and so on]]] 
'         is the value(s) of the constant.
' lambda=<number1>][,<number2>][,<number3>][,and so on]]]
'         is the the value(s) of the autocorrelation coefficient
'         (minus one). 
' sigma=<number1>][,<number2>][,<number3>][,and so on]]]
'         is the the standard deviation(s) of the innovations.
'         By default, sigma=1 (i.e., innivations ~ iid N(0,1)).        

'NOTE: Only one argument of "break", "y0", "var(y0)", "gamma", "const",  "lambda", and "sigma" can be specified as the  list of values!

'EXAMPLES: 
'  file="50nc,across gamma" type=nc break=0.5  N=200000 T=50  y0=0 var(y0)=0 gamma=0,1,2,3,5,10 lambda=0 const=0 
'  file=test  type=c  N=5000 T=50  lambda=0 gamma=10 break=-0.5 const=0 y0=0,0.1,0.5,1 var(y0)=0  
'  type=c break=0.7  file=BAK N=100000 T=100  y0=0 gamma=3 var(y0)=100 lambda=-0.2,-0.1,-0.05,-0.01 const=1
' file="Alt(100,lambda=-01,c=0,b=01,05,09)" T=100 lambda=-0.1 const=0 break=0.1,0.5,0.9 gamma=0 y0=0 var(y0)=0 type=nc N=200000 

' Before starting the program, set "Maximum errors before halting"  
'                                                                       to a number more than 1 ! 

' If workfile already exists on disk, the program cannot change its
'                                parameters (i.e., samle size, equation type, etc.).

'RESULTS:
'CDF_t_nl, CDF_t_P, CDF_DF_nc, and CDF_DF_c are CDFs of t-ratio of the autoregressive coefficient (minus one) in nonlinear test equation, Perron-type equation, and Dickey-Fuller's CDF for their equations without and with constant, respectively. The first column is probability (from 0 to 1 with step 0.001); the rest are critical values by break point. (Thus, the number of column in CDF_t_nl and CDF_t_P is the number of break point + 1.)
'Matrices t_nl, t_P, t_DF_nc, and t_DF_c contain the full set of estimates of the t-ratio of lambda.

'Table DESCRIPTION reports conditions of a given estimation. 

'*************************** SUBROUTINES ***************************
'------------ Find an argument in the program argument list  -------------
Subroutine Find_argument(string %Source, string %List, string %List_length, string %keyword, string %kw_length, string %name, string %argument, string %arg_length, string %Default)
!length=@val(%List_length)
!kw_length=@val(%kw_length)
!first=1
     while !first<!length
              for !last=!first+1 to !length
                    if @mid(%List, !last,1)="*" or %List="" then
                    ExitLoop
                    endif
               next
               if @mid(%List,!first,!kw_length)=%keyword then
                    %arg_length=@str(!last-!first-!kw_length)
                    %argument=@mid(%List,!first+!kw_length,!last-!first-!kw_length)
                    %List=@left(%List,!first-1) + @mid(%List,!last+1)
                    %List_length=@str(!length-(!last-!first)-1)
                    ExitLoop
               else
                    !first=!last+1
               endif
     wend
     if %Default="No" and %argument="" then
          StatusLine No {%name} specified: {%Source}
          stop
     endif
EndSub 'Find_argument

'------------- Compute the numer of values of an argument  -------------
Subroutine Check_value_list(string %argument, string %arg_length, string %keyword, string %maxN_of_parm_values, string %N_of_parm_values)
!k=1
     for !j=1 to @val(%arg_length)
               if @mid(%argument,!j,1)="," then
                    !k=!k+1
               endif
     next
%N_of_parm_values=@str(!k)
     if !k>1 then
               if %maxN_of_parm_values<>"1" then
                    StatusLine Two parameters are specified as the  list of values (the second is {%keyword}{%argument})
                    stop
               else
                    %maxN_of_parm_values=@str(!k)
               endif
     endif
EndSub 'Strip_value_list

'------------- Strip the value list of an argument  -------------
Subroutine Strip_value_list(string %argument, string %arg_length, string %keyword, string %name, vector Value)
!L=@val(%arg_length)
!k=0
!from=1
     for !j=1 to !L+1
               if @mid(%argument,!j,1)="," or !j=!L+1 then
                    !k=!k+1
                    Value(!k)=@val(@mid(%argument,!from,!j-!from))
                        if  Value(!k)=NA then
                             StatusLine Invalid {%name}: {%keyword}{%argument}
                             stop 
                        endif
                    !from=!j+1
               endif
     next
EndSub 'Strip_value_list

'-------------- Transform seconds to hours and minutes ------------------
Subroutine Seconds_to_HM(scalar Seconds, string %Time)
     !minutes=@round(Seconds/60)
     !hours=@floor(!minutes/60)
     !minutes=!minutes-!hours*60
     %Time=""
          if !hours>0 then
               %Time=@str(!hours)+" hours "
          endif
    %Time=%Time+@str(!minutes)+" min. "
EndSub 'Seconds_to_HM

'------------- Compute CDFs of a given set of t-ratio series -------------
Subroutine CDF(matrix Tau, scalar Rows, scalar Cols, matrix CDF, scalar Quantiles)
Series X
     for !i=0 to Quantiles
        CDF(!i+1,1)=!i/Quantiles
     next
!step=Rows/Quantiles
     for !j=1 to Cols
              for !t=1 to Rows
                   X(!t)=Tau(!t,!j)
              next
         Sort X         
         !i=@floor(!step/2-1)
               if !i>0 then
                    CDF(1,!j+1)=X(!i)
                    CDF(Quantiles+1,!j+1)=X(Rows-!i)
               else
                    CDF(1,!j+1)=X(1)-1e-6
                    CDF(Quantiles+1,!j+1)=X(Rows)+1e-6
               endif
               for !i=1 to Quantiles-1
                    CDF(!i+1,!j+1)=X(@ceiling(!i*!step))
              next
     next
delete X
EndSub 'CDF

'****************** PREPARATORY PROCEDURES ******************

'--- READ PROGRAM ARGUMENTS AND FIND WORKFILE NAME ---
tic
%Arguments=%0+"*"+%1+"*"+%2+"*"+%3+"*"+%4+"*"+%5+"*"+%6+"*"+%7+"*"
     if %8<>"" then
          %Arguments=%Arguments+%8+"*"
     endif
     if %9<>"" then
          %Arguments=%Arguments+%9+"*"
     endif
     if %10<>"" then
          %Arguments=%Arguments+%10+"*"
     endif
%Source=%0+" "+%1+" "+%2+" "+%3+" "+%4+" "+%5+" "+%6+" "+%7+" "+%8+ " "+%9+" "+%10
     if %11<>"" then
          StatusLine Superfluous argument(s): ... {%11} {%12} {%13}
          stop
     endif
     if %1="" and @left(%Arguments,5)<>"file=" then
          %Arguments="file="+%Arguments
     endif
     for !i=1 to 4096
            if @mid(%Arguments, !i,1)="" then
            ExitLoop
            endif
     next
%List_length=@str(!i-1)
Call Find_argument(%Source, %Arguments, %List_length, "file=", "5", "workfile name", %file, %a, "No")
!errors=@errorcount
Load {%file}

'---------------------- WORKFILE %0 EXISTS ON DISK. ----------------------
'                                THEN READ ITS PARAMETERS

     if !errors=@errorcount then 'old file
                if %1<>"" then
                    StatusLine The file already exists on disc. Then other arguments are superfluous: {%Source}
                    stop                    
               endif
          !T_=T+1                                                              'Sample size
          !N=N                                                                    'Number of replications
               if Description(4,2)<>"" then
                    SetCell(Description,4,2,@str(Current),"r")
               else
                    StatusLine The estimation has been completed.
                    stop
               endif
     else 'new file 

'---------------------- WORKFILE %0 DOES NOT EXIST. ---------------------
'          THEN CHECK PROGRAM ARGUMENTS, CREATE NEW       
'              WORKFILE %0,  AND CREATE BREAK VARIABLES

'***** Check arguments
          %N_of_parm_values="1"
          Call Find_argument(%Source, %Arguments, %List_length, "T=", "2", "sample size", %T,%a, "No") 
           !T_=@val(%T)
                    if !T_=NA or !T_<5 or @round(!T_)<>!T_ then 
                         StatusLine Invalid samle size: T=%T
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "N=", "2", "number of replications", %N,%a, "No")
           !N=@val(%N)
                    if !N=NA or !N<1 or @round(!N)<>!N then 
                         StatusLine Invalid number of replications: N=%N
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "type=", "5", "type", %type, %a, "No")
                     if %type<>"nc" and %type<>"c" then 
                         StatusLine Invalid type (should be c or nc): type=%type 
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "break=", "6", "break point", %break, %break_length, "No")
          Call  Check_value_list(%break, %break_length, "break=", %N_of_parm_values, %N_of_breaks)

          Call Find_argument(%Source, %Arguments, %List_length, "y0=", "3", "initial value", %y0, %y0_length, "Yes")
                if %y0="" then
                     %y0="0"
                     %y0_length="1"
                     %def_y0=" (by default)"
                else
                     %def_y0=""
                endif
          Call  Check_value_list(%y0, %y0_length, "y0=", %N_of_parm_values, %N_of_means)

          Call Find_argument(%Source, %Arguments, %List_length, "var(y0)=", "8", "variance of y0", %var, %var_length, "Yes")
                if %var="" then
                     %var="0"
                     %var_length="1"
                     %def_var=" (by default)"
                else
                     %def_var=""
                endif
          Call  Check_value_list(%var, %var_length, "var(y0)=", %N_of_parm_values, %N_of_vars)

          Call Find_argument(%Source, %Arguments, %List_length, "gamma=", "6", "break parameter", %gamma,%gamma_length, "No")
          Call  Check_value_list(%gamma, %gamma_length, "gamma=", %N_of_parm_values, %N_of_gammas)

          Call Find_argument(%Source, %Arguments, %List_length, "lambda=", "7", "autoregression parameter", %lambda,%lambda_length, "No")
          Call  Check_value_list(%lambda, %lambda_length, "lambda=", %N_of_parm_values, %N_of_lambdas)

          Call Find_argument(%Source, %Arguments, %List_length, "const=", "6", "break parameter", %const,%const_length, "No")
          Call  Check_value_list(%const, %const_length, "const=", %N_of_parm_values, %N_of_consts)

          Call Find_argument(%Source, %Arguments, %List_length, "sigma=", "6", "s.d. of innivations", %sigma, %sigma_length, "Yes")
                if %sigma="" then
                     %sigma="1"
                     %sigma_length="1"
                     %def_sigma=" (by default)"
                else
                     %def_sigma=""
                endif
          Call  Check_value_list(%sigma, %sigma_length, "sigma=", %N_of_parm_values, %N_of_sigmas)

                if %Arguments<>"" then
                    !length=@val(%List_length)
                         for !i=1 to !length
                              if @mid(%Arguments,!i,1)="*" then
                                   %Arguments=@left(%Arguments,!i-1)
                                   ExitLoop
                              endif
                         next
                    StatusLine Unrecognized argument: {%Arguments}
                    stop                    
               endif

'***** Create new workfile
          !T_=!T_+1
          WorkFile {%file} u 1 !T_
          Scalar T=!T_-1
          Scalar N=!N
          Scalar N_of_parm_values=@val(%N_of_parm_values)
          Scalar N_of_breaks=@val(%N_of_breaks)          
          Scalar N_of_means=@val(%N_of_means)
          Scalar N_of_vars=@val(%N_of_vars)
          Scalar N_of_gammas=@val(%N_of_gammas)
          Scalar N_of_lambdas=@val(%N_of_lambdas)
          Scalar N_of_consts=@val(%N_of_consts)
          Scalar N_of_sigmas=@val(%N_of_sigmas)
          Vector(N_of_breaks) break
          Vector(N_of_breaks) reversed
          Vector(N_of_means) mean_y0
          Vector(N_of_vars) var_y0
          Vector(N_of_gammas) gamma
          Vector(N_of_lambdas) lambda
          Vector(N_of_consts) const
          Vector(N_of_sigmas) sigma
          Call Strip_value_list(%break, %break_length, "break=", "break point(s)", break)
          %breaks=""
               for !b=1 to N_of_breaks
                    reversed(!b)=(break(!b)<0)
                    break(!b)=abs(break(!b))
                    %breaks=%breaks+", "+@str(break(!b))+"*T"
                         if reversed(!b) then
                              %breaks=%breaks+" (reversed)"
                         endif
                    break(!b)=@round(break(!b)*T)
                         if break(!b)=0 or break(!b)>T-1 then
                              close {%file}
                              StatusLine Invalid break argument: breaks=%break
                              stop
                    endif
               next
          %breaks=@mid(%breaks,2)
          Call Strip_value_list(%y0, %y0_length, "y0=", "initial value(s)", mean_y0)
          Call Strip_value_list(%var, %var_length, "var(y0)=", "variance(s) of y0", var_y0)
               for !i=1 to N_of_vars
                    if var_y0(!i)<0 then
                         StatusLine Negative variance: var(y0)=%var
                         stop
                    endif
               next
          Call Strip_value_list(%gamma, %gamma_length, "gamma=", "break heigth(s)", gamma)
          Call Strip_value_list(%lambda, %lambda_length, "lambda=", "autoregression parameter(s)", lambda)
          Call Strip_value_list(%const, %const_length, "const=", "constant", const)
          Call Strip_value_list(%sigma, %sigma_length, "sigma=", "s.d. of innovations", sigma)
               for !i=1 to N_of_sigmas
                    if sigma(!i)<0 then
                         StatusLine Negative standard deviation: sigma=%sigma
                         stop
                    endif
               next
          Scalar Seconds=0
          Scalar Current=1

          Table(12,6) Description
          SetColWidth(Description,1,4)
          SetColWidth(Description,2,15)
          Description(1,1)=%type
               if %type="nc" then 
                    Description(1,2)="Regressions without intercept term"
               else
                    Description(1,2)="Regressions with intercept term"
               endif
          Description(2,1)="Sample size: "+%T
          Description(3,1)="Number of replications: "+%N
          SetCell(Description,4,2,"0","r")
          Description(4,3)="or more performed (see scalar CURRENT for exact figure)"
          Description(5,1)="Break point:  "+%breaks
          Description(6,1)="Break height (gamma): "+%gamma
          Description(7,1)="Constant: "+%const
          Description(8,1)="Autoregression parameter (lambda): "+%lambda
          Description(9,1)="Mean of y0:  "+%y0+%def_y0
          Description(10,1)="Variance of y0: "+%var+%def_var
          Description(11,1)="Standard deviation of the innovations: "+%sigma+%def_sigma
          Description(13)="Started "+@date+" at "+@time
          Description(16)=%Source 
StatusLine %Source

'***** Create break variables (dummies)               
               for !b=1 to N_of_breaks
                     Series Break{!b}=0
                             for !t=break(!b)+2 to !T_
                                  Break{!b}(!t)=1
                             next
                          if reversed(!b) then
                               Break{!b}=1-Break{!b}
                          endif
               next
          Delete break reversed

     endif  'new file

'************************ ESTIMATE MODELS *************************
Series Y
Equation ARnc
Equation ARconst
Equation ARnl_break
Equation ARP_break

'Estimated tsats: "NL" for nonlinear, "P" for Perron-type equation
Matrix(N,N_of_parm_values) t_NL 
Matrix(N,N_of_parm_values) t_P
Matrix(N,N_of_parm_values) t_DF_nc
Matrix(N,N_of_parm_values) t_DF_c
Series eps
   
     if Description(1,1)="nc" then                      'without/with const
         %intercept=""          
     else
         %intercept="+c(2)"
     endif
Show Description
!Backup_time=3600 'seconds, i.e. 1 hour
!Last_backup=@toc
!Elapsed_time=Seconds+@toc
!k1=Current

    for !k=!k1 to N
         Call Seconds_to_HM((Seconds/!k)*(N-!k),%a)
         StatusLine Replication {!k} of {!N}. Estimated time left {%a}
         Current=!k

     'Generate series         
         eps=nrnd
              for !b=1 to N_of_breaks
              for !lb=1 to N_of_lambdas
              for !c=1 to N_of_consts
              for !g=1 to N_of_gammas
              for !m=1 to N_of_means
              for !v=1 to N_of_vars
              for !s=1 to N_of_sigmas             
                    Y(1)=eps(1)*@sqr(var_y0(!v))+mean_y0(!m)
                    smpl @first+1 @last
                    Y=const(!c)+(lambda(!lb)+1)*Y(-1)+gamma(!g)*Break{!b}-(lambda(!lb)+1)*gamma(!g)*Break{!b}(-1)+eps*sigma(!s)
                    smpl @first @last

                    if N_of_breaks>1 then
                         !r=!b
                    endif
                    if N_of_lambdas>1 then
                         !r=!lb
                    endif
                    if N_of_consts>1 then
                         !r=!c
                    endif
                    if N_of_gammas>1 then
                         !r=!g
                    endif
                    if N_of_means>1 then
                         !r=!m
                    endif
                    if N_of_vars>1 then
                         !r=!v
                    endif
                    if N_of_sigmas>1 then
                         !r=!s
                    endif

                     'Dickey-Fuller taus
                     ARnc.ls D(Y)=c(1)*Y(-1)
                     t_DF_nc(!k, !r)=ARnc.@tstats(1)
                     ARconst.ls D(Y)=c(1)*Y(-1)+c(3)
                     t_DF_c(!k, !r)=ARconst.@tstats(1)

                     'Perron-type specification
                     ARP_break.ls D(Y)=c(1)*Y(-1){%intercept}+c(3)*Break{!b}+c(4)*Break{!b}(-1)
                     t_P(!k, !r)= ARP_break.@tstats(1)

                     'Nonlinear specification
                      c(1)=lambda(!lb)
                      c(2)=const(!c)
                      c(3)=gamma(!g)
                      ARnl_break.ls(M=1000,C=1e-07) D(Y)=c(1)*Y(-1){%intercept}+c(3)*(Break{!b}-(c(1)+1)*Break{!b}(-1))
                      t_NL(!k, !r)= ARnl_break.@tstats(1) 
              next '!s
              next '!v
              next '!m
              next '!g
              next '!c
              next '!lb
              next '!b

         !toc=@toc
         Seconds=!Elapsed_time+!toc
               if !toc-!Last_backup>=!Backup_time then
                    SetCell(Description,4,2,@str(Current),"r")
                    save {%file}
                    Show Description
                    !Last_backup=!toc
               endif
    next '!k
Current=Current+1

save {%file}

delete Y eps AR*  
delete  lambda const mean_y0 var_y0 gamma sigma Break*
range 1 !N
smpl @all  
!q=1000 'the number of quantiles
matrix(!q+1,N_of_parm_values+1) CDF_DF_nc
StatusLine Computing Dickey-Fuller CDF (noconst)
call CDF(t_DF_nc, N, N_of_parm_values, CDF_DF_nc, !q)
matrix(!q+1,N_of_parm_values+1) CDF_DF_c
StatusLine Computing Dickey-Fuller CDF (const)
call CDF(t_DF_c, N, N_of_parm_values, CDF_DF_c, !q)
matrix(!q+1,N_of_parm_values+1) CDF_t_NL
StatusLine Computing CDF for nonlinear test
call CDF(t_NL, N, N_of_parm_values, CDF_t_NL, !q)
matrix(!q+1,N_of_parm_values+1) CDF_t_P
StatusLine Computing CDF for Perron-type test
call CDF(t_P, N, N_of_parm_values, CDF_t_P, !q)
range 1 !T_
smpl @all

Description(4,2)=""
Description(4,3)=""
Description(14)="Finished "+@date+" at "+@time
Seconds=!Elapsed_time+@toc
Call Seconds_to_HM(Seconds,%a)
Description(15)="Elapsed time: "+%a
delete Current Seconds T N N_of_*
save {%file}    
StatusLine Finished: N={!N}; elapsed time {%a}

'END OF PROGRAM

