'This program estimates the power of the nonlinear and Perron
'          unit root tests allowing for break, as well as that of the
'          Dickey-Fuller tests, without and with constant,  across
'          simulated time series under user-specified alternative(s):
'                y(t)=const + (lambda+1)*y(t-1) + 
'                      gamma*(B(t,break) - (lambda+1)*B(t-1,break)) + eps(t)
'                eps(t) ~ N(0,1); t = 1,...,T
'          See Instructions.pdf for details.

'          Version: 08.2005 

'          Author: Konstantin Gluschenko
'                       glu@nsu.ru
'                       http://econom.nsu.ru/users/gluschenko

'ARGUMENTS of the program may follow in any order. Only 4 of all the 11arguments are optional, having default values, namely "gamma", "y0", "sigma", and "CVfile". The rest 7 arguments must be specified except for a case when the workfile already already exists on disc (e.g., estimations are processing in parts). The filename only should be provided in such a case (then keyword "file=" may be ommited). 
'By default, the program uses Excel file "CriticalValues.xls" that contains critical values of the tests. User may specify a different relevant file; see Instructions.pdf for its structure.

'The arguments are as follows: 
'(<...> means some value of argument, [...] means the optional part of argument, | means "or", and "and so on" means replication of previous expression):

' file=[<path>]<file_name> is the output workfile name 
'          (if there are blanks, enclose it in quotes).
' T=<number> is the sample size ( t=0,...,T). 
'          (In the workfile and program, it will be t=1,...,T+1.)
' N=<number> is the number of  replications.
' size=<number> is the size of the tests.
' lambda=<number1>,...,<number2>;<number3>
'                |<number1>][,<number2>][,<number3>][,and so on]]]
'          is the list of the values of the autocorrelation coefficient
'          (minus one). The upper form specifies them as a sequence
'          from <number1> through <number2> with increment 
'          "number 3"; the lower one lists any number of specific values.
' const=<number> is the value of the constant.
' break=[-]<number> is the break point as a fraction of the sample 
'          size; minus means a break to be "reversed", that is,  
'          the break dummy=1 if t<=the break point, and 0 otherwise.    
' gamma=<number> is the value of the coefficint on the break 
'          variable. By default, gamma=0.
' y0=<number>|N(<number1>,<number2>|T) is the initial value, 
'          y(0): either as a constant equalling <number> or as a 
'          normal random variable with the mean <number1> and the 
'          variance <number2> or equalling the sample size T. 
'          By default, y0=0.
'sigma=<number> is the standard deviation of the innovations.
'          By default, sigma=1 (i.e., innivations ~ iid N(0,1)).        
' CVfile=[<path>]<file name> is the name of an (Excel) file with
'          critical values of the tests (if there are blanks, enclose it in
'          quotes).
'          By default, CVfile=CriticalValues.xls. (It should be placed to 
'          the default directory; otherwise specify it.) 

'EXAMPLES: 
'  file="test 1" break=0.7 const=0 lambda=0 gamma=0 T=100  y0=N(0.1,6) N=100  size=0.1 CVfile="c:\Projects\UR with break\CV.xls" sigma=0.3
'  size=0.01  N=1000 T=50  file=test  y0=0 const=0.5 gamma=1 break=-0.3 lambda=-0.2,...,0.2;0.1 
'  file="Power(100,g=2,c=0,b=05)" size=0.05  lambda=-0.2,-0.1,-0.05,-0.01,0 break=0.5 N=200000 T=100  const=0

' Before starting the program, set "Maximum errors before halting"  
'                                                                       to a number more than 1 ! 
' If workfile already exists on disk, the program cannot change its
'                   parameters (i.e., samle size, the set of gammas, etc.).

'Table RESULTS reports both conditions of the experiment and its results.

'*************************** SUBROUTINES ***************************
'------------ Find an argument in the program argument list  -------------
Subroutine Find_argument(string %Source, string %List, string %List_length, string %keyword, string %kw_length, string %name, string %argument, string %arg_length, string %Default, string %CheckNum)
!length=@val(%List_length)
!kw_length=@val(%kw_length)
!first=1
     while !first<!length
              for !last=!first+1 to !length
                    if @mid(%List, !last,1)="*" or %List="" then
                    ExitLoop
                    endif
               next
               if @mid(%List,!first,!kw_length)=%keyword then
                    %arg_length=@str(!last-!first-!kw_length)
                    %argument=@mid(%List,!first+!kw_length,!last-!first-!kw_length)
                    %List=@left(%List,!first-1) + @mid(%List,!last+1)
                    %List_length=@str(!length-(!last-!first)-1)
                    ExitLoop
               else
                    !first=!last+1
               endif
     wend
     if %Default="No" and %argument="" then
          StatusLine No {%name} specified: {%Source}
          stop
     endif
!Number=@val(%argument)
     if %CheckNum="Yes" and !Number=NA then
          StatusLine Invalid {%name}: {%keyword}{%argument}
          stop
     endif
EndSub 'Find_argument

'--------- Check a layout data (number of row/column of data)  ---------
Subroutine Check(scalar RC, string %row_or_col, string %Address, string %Text, string %FileName)
               if %row_or_col="col" then
                    !max=256
               else
                    !max=65536 
               endif 
               if RC=NA or RC<1 or RC>!max or @round(RC)<>RC then
                    StatusLine Error in cell {%Address} of {%FileName} {%Text}
                    stop
               endif
EndSub 'Check

'---- Check whether there are critical values for a given parameter ---
Subroutine Check_availability(matrix Parameter, scalar Scale, scalar Length, string %Name, string %Text, scalar Value, scalar place)
     Parameter(Length+1,1)=NA
          for place=1 to Length
               if Value<=Parameter(place,1) then
                         ExitLoop
               endif      
          next
     !high=Parameter(place,1)
          if place>1 then 
               !low=Parameter(place-1,1)
          else
               !low=NA
          endif 
          if Parameter(place,1)<>NA then
               Parameter(place,1)=@round(Parameter(place,1)*Scale)/Scale
          endif
          if Value<>Parameter(place,1) or place>Length then
                 place=0
                 %Text="No statistics for "+%Name+" "+@str(Value)+". The nearest values are  "+@str(!low)+" and "+@str(!high)
          endif
EndSub 'Check_availability

'------------- Compute lettering of a column from its number -------------
Subroutine Column(scalar Column, string %Column)
          %ABC="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
          %Column=""
          !f=@floor(Column/26-0.03)
               if !f>0 then
                    %Column=%Column+@mid(%ABC,!f,1)
               endif
          !s=Column-!f*26
          %Column=%Column+@mid(%ABC,!s,1)
EndSub 'Column

'-------------------------- Process non-critical error --------------------------
Subroutine Error(scalar MessageRow, string %Message)
          Results(20+!Number_of_lambdas,1)="ERRORS/WARNINGS:"
          Results(MessageRow,1)=%Message 
          MessageRow=MessageRow+1
EndSub 'Error

'-------------- Transform seconds to hours and minutes ------------------
Subroutine Seconds_to_HM(scalar Seconds, string %Time)
     !minutes=@round(Seconds/60)
     !hours=@floor(!minutes/60)
     !minutes=!minutes-!hours*60
     %Time=""
          if !hours>0 then
               %Time=@str(!hours)+" hours "
          endif
    %Time=%Time+@str(!minutes)+" min. "
EndSub 'Seconds_to_HM

'****************** PREPARATORY PROCEDURES ******************

'--- READ PROGRAM ARGUMENTS AND FIND WORKFILE NAME ---
tic
%Arguments=%0+"*"+%1+"*"+%2+"*"+%3+"*"+%4+"*"+%5+"*"+%6+"*"
     if %7<>"" then
          %Arguments=%Arguments+%7+"*"
     endif
     if %8<>"" then
          %Arguments=%Arguments+%8+"*"
     endif
     if %9<>"" then
          %Arguments=%Arguments+%9+"*"
     endif
     if %10<>"" then
          %Arguments=%Arguments+%10+"*"
     endif
%Source=%0+" "+%1+" "+%2+" "+%3+" "+%4+" "+%5+" "+%6+" "+%7+" "+%8+" "+%9+" "+%10
     if %11<>"" then
          StatusLine Superfluous argument(s): ... {%11} {%12} {%13}
          stop
     endif
     if %1="" and @left(%Arguments,5)<>"file=" then
          %Arguments="file="+%Arguments
     endif
     for !i=1 to 4096
            if @mid(%Arguments, !i,1)="" then
            ExitLoop
            endif
     next
%List_length=@str(!i-1)
Call Find_argument(%Source, %Arguments, %List_length, "file=", "5", "workfile name", %file, %a, "No", "No")
!errors=@errorcount
Load {%file}

'------------------------ WORKFILE EXISTS ON DISK. ------------------------
'                                 THEN READ ITS PARAMETERS

     if !errors=@errorcount then 'old file
                if %1<>"" then
                    StatusLine The file already exists on disc. Then other arguments are superfluous: {%Source}
                    stop                    
               endif
          !T_=T+1                       'Sample size
          !N=N                             'Number of replications
               if Results(3,2)<>"" then
                    SetCell(Results,3,2,@str(Current),"r")
               else
                    StatusLine The estimation has been completed.
                    stop
               endif
     else 'new file 

'------------------------ WORKFILE DOES NOT EXIST. -----------------------
'   THEN CHECK PROGRAM ARGUMENTS AND CREATE NEW WORKFILE

'***** Check arguments
          Call Find_argument(%Source, %Arguments, %List_length, "T=", "2", "sample size", %T,%a, "No", "No") 
           !T_=@val(%T)
                    if !T_=NA or !T_<1 or @round(!T_)<>!T_ then 
                         StatusLine Invalid samle size: T={%T}
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "N=", "2", "number of replications", %N,%a, "No", "No")
           !N=@val(%N)
                    if !N=NA or !N<1 or @round(!N)<>!N then 
                         StatusLine Invalid number of replications: N={%N}
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "size=", "5", "test size", %test_size, %a, "No", "Yes")
           !test_size=@val(%test_size)

          Call Find_argument(%Source, %Arguments, %List_length, "y0=", "3", "initial value", %y0, %a, "Yes", "No")
                if %y0="" then
                     %y0="0"
                     %def_y0=" (by default)"
                else
                     %def_y0=""
                endif
                if @left(%y0,2)<>"N(" then                                       'y(0)=const
                     !mean_y0=@val(%y0)
                     !var_y0=0
                else                                                                             'y(0)~N(m,v)
                     !l=@val(%a)
                          for !i=3 to !l-1
                               if @mid(%y0,!i,1)="," then
                               exitloop
                               endif
                          next
                     !mean_y0=@val(@mid(%y0,3,!i-3))
                     !var_y0=@val(@mid(%y0,!i+1,!l-!i-1))
                          if @right(%y0,!l-!i)="T)" then
                               !var_y0=!T_
                          endif
                          if @right(%y0,1)<>")" then
                               !var_y0=NA
                          endif
                endif
                if !mean_y0=NA or !var_y0=NA or !var_y0<0 then
                     StatusLine Invalid initial condition: y(0)={%y0}
                     stop
                endif

          Call Find_argument(%Source, %Arguments, %List_length, "const=", "6", "constant", %const,%a, "No", "Yes")
           !const=@val(%const)

          Call Find_argument(%Source, %Arguments, %List_length, "break=", "6", "break point", %break, %a, "No", "Yes")
           !break=@val(%break)
           !reversed=(!break<0)
           !break=abs(!break)
                if !break=0 or !break>=1 then
                     StatusLine Invalid break point: break={%break}
                     stop
                endif
           !break=@round(!break*!T_)/!T_

          Call Find_argument(%Source, %Arguments, %List_length, "gamma=", "6", "break height", %gamma,%a, "Yes", "No")
                if %gamma="" then
                     %gamma="0"
                     %def_gamma=" (by default)"
                else
                     %def_gamma=""
                endif
           !gamma=@val(%gamma)
                    if !gamma=NA then 
                         StatusLine Invalid break height: gamma={%gamma}
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "lambda=", "7", "autoregression parameter(s)", %lambda, %a, "No", "No")
          !Nl=1
          !Number_of_lambdas=NA
          !l=@val(%a)
          !j=0
               for !i=2 to !l
                    if @mid(%lambda,!i,5)=",...," then
                         !j=!i
                    endif
                    if @mid(%lambda,!i,1)=";" then
                         ExitLoop
                    endif
               next
               if !j<!l-6 and !j>1 and !i<!l then                     'lambda=l1,...,ln;d
                    !from=@val(@left(%lambda,!j-1))
                    !to=@val(@mid(%lambda,!j+5,!i-(!j+5)))
                    !by=@val(@mid(%lambda,!i+1,!l-!i))
                    !Nl=(!to-!from)/!by+1+10e-8
                       if  !from=NA or !to=NA or !by=NA or !Nl<2 then
                           StatusLine Invalid autoregression parameter(s): lambda={%lambda}
                           stop
                       endif 
                    !Number_of_lambdas=@round(!Nl)
                       if @abs(!Number_of_lambdas-!Nl)>10e-7 then
                           StatusLine Non-integer number of lambdas: (!to-!from)/!by=!Nl
                           stop
                       endif
                    !lambda=!from-!by 
               endif
               if !Number_of_lambdas=NA then           'lambda=l1,l2,l3,etc.
                    !Number_of_lambdas=1
                         for !i=1 to !l
                               if @mid(%lambda,!i,1)="," then
                                    !Number_of_lambdas=!Number_of_lambdas+1
                               endif
                         next 
               endif

          Call Find_argument(%Source, %Arguments, %List_length, "sigma=", "6", "s.d. of innovations", %sigma,%a, "Yes", "No")
                if %sigma="" then
                     %sigma="1"
                     %def_sigma=" (by default)"
                else
                     %def_sigma=""
                endif
           !sigma=@val(%sigma)
                    if !sigma=NA then 
                         StatusLine Invalid s.d. of innovations: sigma={%sigma}
                         stop
                    endif

          Call Find_argument(%Source, %Arguments, %List_length, "CVfile=", "7", "c.v. file", %CVfile,%a, "Yes", "No")
                if %CVfile="" then
                     %CVfile="CriticalValues.xls"
                     %def_cvfile=" (by default)"
                else
                     %def_cvfile=""
                endif

                if %Arguments<>"" then
                    !length=@val(%List_length)
                         for !i=1 to !length
                              if @mid(%Arguments,!i,1)="*" then
                                   %Arguments=@left(%Arguments,!i-1)
                                   ExitLoop
                              endif
                         next
                    StatusLine Unrecognized argument: {%Arguments}
                    stop                    
               endif

'***** Create new workfile
          !T_=!T_+1
          WorkFile {%file} u 1 !T_
          Scalar T=!T_-1
          Scalar mean_y0=!mean_y0
          Scalar var_y0=!var_y0
          Scalar N=!N
          Scalar Number_of_lambdas=!Number_of_lambdas
          Matrix(Number_of_lambdas,6) Power=0
          Scalar const=!const
          Scalar gamma=!gamma
          Scalar sigma=!sigma
          Scalar Seconds=0
          Scalar Current=1
 
          Vector(!Number_of_lambdas) lambda
          !i=0
               for !lb=1 to !Number_of_lambdas
                    if !Nl>1 then
                         !lambda=!lambda+!by
                         !lambda=@round(!lambda*10e7)/10e7
                         lambda(!lb)=!lambda
                    else
                              for !j=!i+1 to !l
                                   if @mid(%lambda,!j,1)="," then
                                   ExitLoop
                                   endif
                              next
                        lambda(!lb)=@val(@mid(%lambda,!i+1,!j-!i-1))
                         !i=!j
                    endif
                    if lambda(!lb)=NA then
                         close {%file}
                          StatusLine Invalid autoregression parameter(s): lambda={%lambda}
                         stop
                    endif
             next '!lb

'***** Create the table of results
          Table(30+!Number_of_lambdas,10) Results
               for !i=2 to 7
                     SetColWidth(Results,!i,13)
               next
          Results(1,1)="POWER EXPERIMENT"
          Results(2,1)="Number of replications: "+%N
          SetCell(Results,3,1,"0","r")
          Results(3,2)="or more performed (see scalar CURRENT for exact figure) "
          Results(4,1)="Sample size, T: "+%T
          Results(5,1)="Break point:  "+@str(!break)+"*T"
               if !reversed then
                    Results(5,1)=Results(5,1)+", reversed break (111...000)"
               endif
          Results(6,1)="Break height, gamma: "+%gamma+%def_gamma
          Results(7,1)="Constant: "+%const
               if const<>0 then
                    Results(7,1)=Results(7,1)+" (= 0 with lambda = 0)"
               endif
               if !var_y0=0 then 
                    Results(8,1)="Initial condition: y(0) = "+%y0+%def_y0
               else
                    Results(8,1)="Initial condition: y(0) ~ "+%y0
               endif
          Results(9,1)="Standard deviation of the innovations, sigma: "+%sigma+%def_sigma
          Results(11,1)="SIZE of the tests: "+%test_size
          Results(11,3)="Critical values from: "+%CVfile+%def_cvfile
          Results(12,1)="Test:"
          Results(12,2)="DF tau-0"
          Results(12,3)="DF tau-mu"
          Results(12,4)="Nonlin. tau-0"
          Results(12,5)="Nonlin. tau-mu"
          Results(12,6)="Perron tau-0"
          Results(12,7)="Perron tau-mu"
          SetLine(Results,13)
          Results (14,1)=@str(!test_size*100)+"% c.v.:"
          SetLine(Results,15)
          Results(16,1)="lambda"
          Results(16,4)="ESTIMATED POWER:"
               for !lb=1 to !Number_of_lambdas
                    Results(16+!lb,1)=@str(lambda(!lb))
               next
          Results(17+!Number_of_lambdas,1)="Started "+@date+" at "+@time
          Results(20+!Number_of_lambdas,1)=%Source
 
'***** Read critical values of the tests
          Vector(6) Critical_value
          Matrix(6,1) Layout
     'Read map of the file 
          !errors=@errorcount
          Layout.read(rect,s=Map,B2)  {%CVfile}
               if !errors<>@errorcount then
                    StatusLine Error while reading {%CVfile} or it does not contain a sheet named Map
                    stop
               endif
          !p_value_col=Layout(1,1)
          Call Check(!p_value_col, "col", "Map!B2", "(column of p-values)", %CVfile)
          !Break_points_row=Layout(2,1)
          Call Check(!Break_points_row, "row", "Map!B3", "(row of break points)", %CVfile)
          !No_of_sampe_sizes=Layout(3,1)
          Call Check(!No_of_sampe_sizes, "col", "Map!B4", "(the number of samle sizes)", %CVfile)
          !Data_beginning=Layout(4,1)
          Call Check(!Data_beginning, "row", "Map!B5", "(first row of the data)", %CVfile)
          !Data_end=Layout(5,1)
          Call Check(!Data_end, "row", "Map!B6", "(last row of the data)", %CVfile)
               if !Data_end<!Data_beginning then
                    StatusLine The numer of the first row of the data is smaller than that of the last row (cells Map!B5 and B6 of {%CVfile})
                    stop
               endif
          !Allocation_table_row=Layout(6,1)
          Call Check(!Allocation_table_row, "row", "Map!B7", "(first row of the allocation table)", %CVfile)
          Delete Layout
          Matrix(!No_of_sampe_sizes+1,1) Sample_size
          Sample_size.read(rect,s=Map,A{!Allocation_table_row})  {%CVfile}
          !r=0
          %Message=""
          Call Check_availability(Sample_size, 1, !No_of_sampe_sizes, "sample size", %Message,T, !r)
                if !r=0 then
                     StatusLine {%Message}
                     stop
                endif
          Delete Sample_size

     'Read data allocation table
          Matrix(1,14) CVlayout
          !r=!Allocation_table_row+!r-1
          %r=@str(!r) 
          CVlayout.read(rect,s=Map,B{!r})  {%CVfile}
          Vector(6) Start
          Vector(6) End
          Start(1)=CVLayout(1,13)                 'DF nc
          Start(2)=CVLayout(1,14)                 'DF c
               if !reversed then
                    Start(3)=CVLayout(1,3)         'Nonlin. nc, reversed break
                    End(3)=CVLayout(1,4)       
                    Start(5)=CVLayout(1,9)         'Perron nc, reversed break
                    End(5)=CVLayout(1,10)
                    %ATcolumn="N O DEFGJKLM"
                    Results(12,3+1)=Results(12,3+1)+"(r)"
                    Results(12,5+1)=Results(12,5+1)+"(r)"
               else
                    Start(3)=CVLayout(1,1)         'Nonlinear noconst
                    End(3)=CVLayout(1,2)       
                    Start(5)=CVLayout(1,7)         'Perron noconst
                    End(5)=CVLayout(1,8)
                    %ATcolumn="N O BCFGHILM"
               endif
          Start(4)=CVLayout(1,5)                   'Nonlinear const
          End(4)=CVLayout(1,6)
          Start(6)=CVLayout(1,11)                 'Perron const
          End(6)=CVLayout(1,12) 
          Delete CVlayout

     'Read critical values
          Matrix(!Data_end-!Data_beginning+2,1) Test_sizes
          Matrix(1,1) CV 
          !e=21+!Number_of_lambdas
               for !i=1 to 6
                    if Start(!i)=NA or Start(!i)<1 or Start(!i)>256 or @round(Start(!i))<>Start(!i) then
                        Call Error(!e, "No "+Results(12,!i+1)+" statistic for T="+@str(T)+" or invalid number of its first column (cell Map!"+@mid(%ATcolumn, !i*2-1,1)+%r+" of "+%CVfile+")")
                        Critical_value(!i)=NA
                    else
                         if !i>2 and (End(!i)=NA or End(!i)<1 or End(!i)>256 or @round(End(!i))<>End(!i) or End(!i)-Start(!i)<0) then
                              Call Error(!e,"Absent or invalid number of the last column of "+Results(12,!i+1)+" (cell Map!"+@mid(%ATcolumn, !i*2,1)+%r+" of "+%CVfile+")") 
                             Critical_value(!i)=NA
                         endif
                    endif
                    if Critical_value(!i)<>NA then
                         Call Column(!p_value_col,%Column)
                         %sheet="s="+Results(12,!i+1)
                         !errors=@errorcount
                         Test_sizes.read(rect,{%sheet},{%Column}{!Data_beginning})  {%CVfile}
                              if !errors<>@errorcount then
                                   Call Error(!e,"File "+%CVfile+" does not contain a sheet named "+Results(12,!i+1))
                                   Critical_value(!i)=NA
                              else
                                  Call Check_availability(Test_sizes, 1000000, !Data_end-!Data_beginning+1, "test size",%Message, !test_size, !r)
                                        if !r=0 then
                                             Call Error(!e, Results(12,!i+1)+": "+%Message)
                                             Critical_value(!i)=NA
                                        else
                                             !r=!Data_beginning+!r-1
                                        endif
                              endif
                    endif
                    if !i>2 and Critical_value(!i)<>NA then
                         !c=0
                         Matrix(1,End(!i)-Start(!i)+2) BreakPoints
                         Call Column(Start(!i),%Column)
                         BreakPoints.read(rect,{%sheet},{%Column}{!Break_points_row})  {%CVfile}
                         Matrix Break_Points=@transpose(BreakPoints)
                         Call Check_availability(Break_Points, T, End(!i)-Start(!i)+1, "break point",%Message, !break, !c)
                              if !c=0 then
                                   Call Error(!e, Results(12,!i+1)+": "+%Message)
                                   Critical_value(!i)=NA
                              endif
                    endif
                    if Critical_value(!i)<>NA then
                              if !i=1 or !i=2 then
                                   Call Column(Start(!i),%Column) 
                              else
                                   Call Column(Start(!i)+!c-1,%Column)
                              endif 
                         CV.read(rect,{%sheet},{%Column}{!r})  {%CVfile}
                         Critical_value(!i)=CV(1,1)
                    endif
                    if Critical_value(!i)=NA then
                         for !j=1 to Number_of_lambdas
                              Power(!j,!i)=NA
                         next
                    endif
                   SetCell(Results,14,!i+1,Critical_value(!i),3)
               next
          Results(12,4)="Nonlin. tau-0"
          Results(12,6)="Perron tau-0"
          Delete  Start End Test_sizes CV 
               if @IsObject("BreakPoints") then
                    Delete BreakPoints Break_Points
               endif 

'***** Create break dummy
          Series Break=0     
               for !t=!break*T+2 to !T_
                      Break(!t)=1
               next
               if !reversed then
                    Break=1-Break
               endif
     endif  'new file

'****************** ESTIMATE AND TEST MODELS *******************
Series Y
Series eps
Equation ARnc
Equation ARc
Equation ARP_nc
Equation ARP_c
Equation ARnl_nc
Equation ARnl_c

Show Results
!Backup_time=3600 'seconds, i.e. 1 hour
!Last_backup=@toc
!Elapsed_time=Seconds+@toc
!k1=Current

    for !k=!k1 to N
         Call Seconds_to_HM((Seconds/!k)*(N-!k),%a)
         StatusLine Replication {!k} of {!N}. Estimated time left {%a}
         Current=!k

         'Generate innivations and the initail value of a time series
         eps=nrnd
         Y(1)=eps(1)*@sqr(var_y0)+mean_y0

              for !lb=1 to Number_of_lambdas

               'Generate a time series 
                   smpl @first+1 @last
                   !rho=lambda(!lb)+1
                        if lambda(!lb)=0 then
                             !const=0
                        else
                             !const=const
                        endif
                   Y=!const+!rho*Y(-1)+gamma*Break-!rho*gamma*Break(-1)+eps*sigma
                   smpl @first @last

               'Dickey-Fuller test
                   ARnc.ls D(Y)=c(1)*Y(-1)
                   Power(!lb,1)=Power(!lb,1)+(@tstats(1)<=Critical_value(1))/N
                   ARc.ls D(Y)=c(1)*Y(-1)+c(2)
                   Power(!lb,2)=Power(!lb,2)+(@tstats(1)<=Critical_value(2))/N

               'Nonlinear test
                   c(1)=lambda(!lb)
                   c(3)=gamma
                   ARnl_nc.ls(M=1000,C=1e-07)  D(Y)=c(1)*Y(-1)+c(3)*(Break-(c(1)+1)*Break(-1))
                   Power(!lb,3)=Power(!lb,3)+(@tstats(1)<=Critical_value(3))/N
                   c(1)=lambda(!lb)
                   c(2)=!const
                   c(3)=gamma
                   ARnl_c.ls(M=1000,C=1e-07)  D(Y)=c(1)*Y(-1)+c(2)+c(3)*(Break-(c(1)+1)*Break(-1))
                   Power(!lb,4)=Power(!lb,4)+(@tstats(1)<=Critical_value(4))/N

               'Perron test
                   ARP_nc.ls D(Y)=c(1)*Y(-1)+c(3)*Break+c(4)*Break(-1)
                   Power(!lb,5)=Power(!lb,5)+(@tstats(1)<=Critical_value(5))/N
                   ARP_c.ls D(Y)=c(1)*Y(-1)+c(2)+c(3)*Break+c(4)*Break(-1)
                   Power(!lb,6)=Power(!lb,6)+(@tstats(1)<=Critical_value(6))/N
              next '!lb

         !toc=@toc
         Seconds=!Elapsed_time+!toc
               if !toc-!Last_backup>=!Backup_time then
                    SetCell(Results,3,1,@str(Current),"r")
                         for !i=1 to 6
                         for !lb=1 to Number_of_lambdas
                              SetCell(Results,16+!lb,!i+1,Power(!lb,!i)*N/Current,3)
                         next
                         next 
                    save {%file}
                    Show Results
                    !Last_backup=!toc
               endif
    next '!k

Results(3,1)=""
Results(3,2)=""
     for !i=1 to 6
     for !lb=1 to Number_of_lambdas
           SetCell(Results,16+!lb,!i+1,Power(!lb,!i),3)
     next
     next
Results(18+Number_of_lambdas,1)="Finished "+@date+" at "+@time
Seconds=!Elapsed_time+@toc
Call Seconds_to_HM(Seconds,%a)
Results(19+Number_of_lambdas,1)="Elapsed time: "+%a
  
Delete Current Seconds Number_of_lambdas 
Delete Y eps    
Delete AR* 
Delete T N Break mean_y0 var_y0 const gamma lambda sigma Critical_value

Save {%file}
StatusLine Finished: N={!N}. Elapsed time {%a}

'END OF PROGRAM

